/* BN.C
 * SBC++ (soccer client for Robocup2000)
 * Eslam Nazemi<nazemi@cc.sbu.ac.ir>
 * Amin Abbaspour<a-abbaspour@ce.sbu.ac.ir>
 * Mahmood Rahmani<m-rahmani@ce.sbu.ac.ir>
 * Bahman Radjabalipoor<b-radjabalipoor@ce.sbu.ac.ir>
 * Computer Department
 * Shahid Beheshti University
 */

#include "BN.h"
#include "NetHeader.h"
#include "NetFuncs.h"
#include "ParsTools.h"
#include "NetParams.h"
#include "utils.h"
#include "client.h"

#include <math.h>

#define DEFAULT_LEG_TH	 	0.05
#define DEFAULT_MOUTH_TH 	0.4
#define DEFAULT_NECK_TH  	0.2

#define DEFAULT_LEG_TH_RED	0.05
#define DEFAULT_MOUTH_TH_RED 	0.1
#define DEFAULT_NECK_TH_RED  	0.1

TBehaviorNetwork::TBehaviorNetwork(const char *sFileName)
		 :TParseTools( sFileName )
{
    DEBUG(printf("TBehaviorNetwork : Starting...\n"));

    iCompIndex 	= 0;
    iGoalIndex 	= 0;

    ResTh[RESOURCE_LEG]	  = DefResTh[RESOURCE_LEG]   = DEFAULT_LEG_TH;
    ResTh[RESOURCE_NECK]  = DefResTh[RESOURCE_NECK]  = DEFAULT_NECK_TH;
    ResTh[RESOURCE_MOUTH] = DefResTh[RESOURCE_MOUTH] = DEFAULT_MOUTH_TH;

    DefResThRed[RESOURCE_LEG]  	= DEFAULT_LEG_TH_RED;
    DefResThRed[RESOURCE_LEG]  	= DEFAULT_NECK_TH_RED;
    DefResThRed[RESOURCE_LEG] 	= DEFAULT_MOUTH_TH_RED;

    pComp 	= new TCompetence [ GetCompBlockNum() ];
    pGoal 	= new TGoal       [ GetGoalBlockNum() ];

    if(!pComp || !pGoal)
    {
        my_error("TBehaviorNetwork : Memory allocation faliure.\n");
        _exit(0);
    }

    ParseRuleFile();
    AdjustNetworkLinks();

    DEBUG(printf("TBehaviorNetwork : Done.\n"));
}

TBehaviorNetwork::~TBehaviorNetwork()
{
    delete [] pGoal;
    delete [] pComp;
}

void TBehaviorNetwork::ParseRuleFile()
{
    DEBUG(printf("ParseRuleFile : Starting...\n"));
    while( ReadNextBlock() )
	switch( BlockType() )
	{
	    case BLOCK_COMPETENCE:
	    {
		DEBUG(printf("ParseRuleFile : Parse Comp. Block %d.\n",iCompIndex));
		ParseCompetence( pComp[iCompIndex++] );
		DEBUG(printf("ParseRuleFile : Comp. Block %d Parsed.\n",iCompIndex-1));
		break;
	    }

	    case BLOCK_GOAL:
	    {
		DEBUG(printf("ParseRuleFile : Parse Goal. Block %d.\n",iGoalIndex));
		ParseGoal( pGoal[iGoalIndex++] );
		DEBUG(printf("ParseRuleFile : Goal. Block %d Parsed.\n",iGoalIndex-1));
		break;
    	    }

	    default :
	    {
		my_error("ParseRuleFile : Passing an unkown block.\n");
		break;
	    }
	}
    CloseFile();
    DEBUG(printf("ParseRuleFile : Done.\n"));
}

void TBehaviorNetwork::AdjustNetworkLinks()
{
    DEBUG(printf("AdjustNetworkLinks : Starting...\n"));
    
    for(int i = 0; i < iCompIndex; i++)
    	for(int j = 0;  j < iCompIndex; j++)
	    if(i != j)
		for(int x = 0; x < pComp[i].GetEffIndex(); x++)
		    for(int y = 0; y < pComp[j].GetPreIndex(); y++)
			if(pComp[i].rEff[x]->GetFunc() == pComp[j].rPre[y]->GetFunc())
			{
			    float val = pComp[i].rEff[x]->GetExp();
			    if(val * pComp[j].rPre[y]->GetExp() < 0 ) val *= -1;
			    pComp[i].AddExpantion(j, pComp[i].rEff[x]->GetFunc(),val);
			}
    DEBUG(printf("AdjustNetworkLinks : Done.\n"));
}

void TBehaviorNetwork::ShowNetwork()
{
    for(int i = 0; i < iCompIndex; i++)
    {
	printf("Comp. No : %d\n",i);
	ShowCompetence( pComp[i] );
        printf( NEW_LINE );
    }
}

int TBehaviorNetwork::GetGoalNumOfComp(TCompetence aComp)
{
    int iGoal = 0;
    for(int i = 0; i < aComp.GetPreIndex(); i++)
	if( aComp.rEff[i]->GetFunc() > GoalCommandStart ) iGoal++;

    return ( iGoal + aComp.GetExpIndex() );
}

int TBehaviorNetwork::GetGoalFunc(const int& FuncNum)
{
    TGoal *aGoal;
    for(int i = 0; i < iGoalIndex; i++) {
	aGoal = &pGoal[i];
        if( aGoal->GetFunc() == FuncNum ) return aGoal->Percept->GetFunc();
    }
    return 0;
}

int TBehaviorNetwork::GetGoalNum(const int& FuncNum)
{
    TGoal *aGoal;
    for(int i = 0; i < iGoalIndex; i++) {
	aGoal = &pGoal[i];
        if( aGoal->GetFunc() == FuncNum ) return i;
    }
    return 0;
}

void TBehaviorNetwork::PreExecuteNetwork()
{
    DEBUG(printf("PreExecuteNetwork : Starting...\n"));
    RunAllTruePercept();
    RunAllGoals();
    DEBUG(printf("PreExecuteNetwork : Done.\n"));
}

void TBehaviorNetwork::RunAllGoals()
{
    DEBUG(printf("RunAllGoals : Goals executing.\n"));
    for(int i = 0; i < iGoalIndex; i++)  {
	pGoal[i].SetOutput( 0 );

	if( IsTrue ( GetFuncValue(pGoal[i].Percept->GetFunc(),
		     pGoal[i].Percept->GetExp() ) ) ) {
	    
	    if( !IsTrue( GetFuncValue( pGoal[i].GetFunc(), pGoal[i].GetImp() ) ) )
		pGoal[i].SetOutput( GetFuncValue( pGoal[i].GetFunc(), -pGoal[i].GetImp()) );
	}
    }
    DEBUG(printf("RunAllGoals : Done.\n"));
}

void TBehaviorNetwork::CalcCompExpect()
{
    float       swpExe;
    float	swpOut;
    int 	i, j;    
    DEBUG(printf("CalcCompExpect : Calculating Expectaion.\n"));
    for(i = 0; i < iCompIndex; i++)
    {
	swpExe = 1.0;

	for(j = 0; j < pComp[i].GetPreIndex(); j++) {
	    swpOut = GetFuncValue( pComp[i].rPre[j]->GetFunc(),
				    pComp[i].rPre[j]->GetExp());
            if(swpOut < swpExe) swpExe = swpOut;
				    
            if(swpOut < swpExe) swpExe = swpOut;
	}
	pComp[i].SetExec(swpExe);

	for(j = 0; j < pComp[i].GetGoalIndex(); j++) {
	    pComp[i].SetPrevActOfGoal(j, pComp[i].GetActOfGoal(j));
	    pComp[i].SetActOfGoal(j, -1000.0);
	}
    }
    DEBUG(printf("CalcCompExpec : Done.\n"));
}

void TBehaviorNetwork::DoActivationSpreading()
{
    DEBUG(printf("DoActivationSpreading : Start...\n"));

    int 	i, j, a; 
    float 	fSigma;
    int         goalIndex = 0;
    float 	val;
                
    PreExecuteNetwork();
    CalcCompExpect();

    for(i = 0; i < iCompIndex; i++) {
	goalIndex = 0;

    for(j = 0; j < pComp[i].GetEffIndex(); j++) 
	if( pComp[i].rEff[j]->GetFunc()  > GoalCommandStart) {
	    
	    val = pGoal[GetGoalNum(pComp[i].rEff[j]->GetFunc())].GetOutput() * 
		    pComp[i].rEff[j]->GetExp();

	    if( val > 0 ) 
        	pComp[i].SetActOfGoal(goalIndex, GetGama() * val);
	    else 
        	pComp[i].SetActOfGoal(goalIndex, GetDelta() * val);

	    goalIndex++;
	}

    for(j = 0; j < pComp[i].GetExpIndex(); j++)
	if( pComp[i].rExp[j]->GetExp() > 0)
	    for(a = 0; a < pComp[i].GetGoalIndex(); a++)
	        pComp[i].SetActOfGoal(a,
		    GetGama() *
		    Goetz(pComp[pComp[i].rExp[j]->iComp].GetPrevActOfGoal(a) )*
		    pComp[i].rEff[j]->GetExp() *
		    GetFuncValue( pComp[i].rExp[j]->GetFunc(), -1 )
		    );

	else
	    for(a = 0; a < pComp[i].GetGoalIndex(); a++)
    		pComp[i].SetActOfGoal(a,
		    GetDelta() * 
		    Goetz(pComp[pComp[i].rExp[j]->iComp].GetPrevActOfGoal(a) )*
		    pComp[i].rEff[j]->GetExp() *
		    GetFuncValue( pComp[i].rExp[j]->GetFunc() )
		    );

	fSigma = 0.0;
	for(j = 0; j < pComp[i].GetGoalIndex(); j++) {
		fSigma += pComp[i].GetActOfGoal(j);
	}
	pComp[i].SetAct( GetBeta() * pComp[i].GetPrevAct() + fabs(fSigma));
    } 
    DEBUG(printf("DoActivationSpreading : Done.\n"));
}

void TBehaviorNetwork::ExecuteNetwork(const int& iUnusedRes = -1)
{
    int    i, j;
    float  ResSwp[MAX_RESOURCE] = {0.0, 0.0, 0.0};    
    int    ResF[MAX_RESOURCE] =   {  0,   0,   0};    
        
    DoActivationSpreading();
    
    for(i = 0; i < iCompIndex; i++) {
        if( H(pComp[i].GetAct(), pComp[i].GetExec())  > ResSwp[pComp[i].GetResource()] )	{
	    ResSwp[pComp[i].GetResource()] = H( pComp[i].GetAct(), pComp[i].GetExec() );
	    ResF  [pComp[i].GetResource()] = pComp[i].GetFunc();
	}
    }

//    DebugIt();
    Mem->LogAction2(1,"Behavior Network:");
    
    for(j = 0; j < MAX_RESOURCE; j++) 
	if(j != iUnusedRes) {
    	    if( ResSwp[j] >= ResTh[j]) {
    		ResTh[j] = DefResTh[j];
		RunBehave( ResF[j] );
	    }
	    else {
		ResTh[j] -= DefResThRed[j];
    	    }	    
	}

    for(i = 0; i < iCompIndex; i++) pComp[i].SetPrevAct( pComp[i].GetAct() );
}

void TBehaviorNetwork::DebugIt()
{
    int i;
    float val;
    
    for(i = 0; i < iCompIndex; i++) {
    	    printf("* %s,\tE = %.3f,A = %.3f,H = %.3f:",
		pComp[i].GetName(),
		pComp[i].GetExec(), pComp[i].GetAct(), H( pComp[i].GetAct(), pComp[i].GetExec()));

	    for(int j = 0; j < pComp[i].GetPreIndex(); j++)	{
		float val = GetFuncValue(pComp[i].rPre[j]->GetFunc()
					    ,pComp[i].rPre[j]->GetExp() );
		printf("e%d=%1.2f, ",j,val); 
	    }

	    printf("\t");

	    for(int j = 0; j < pComp[i].GetEffIndex(); j++)	{
		if( pComp[i].rEff[j]->GetFunc() > GoalCommandStart) {
		    val = pGoal[GetGoalNum(pComp[i].rEff[j]->GetFunc())].GetOutput();
    		    printf("ag%d=%1.2f, ",j,val);
		}
		else {
		    val = GetFuncValue(pComp[i].rEff[j]->GetFunc(), 
			    pComp[i].rEff[j]->GetExp()) ;
    		    printf("ap%d=%1.2f, ",j,val);

		}
	    }
	    printf(NEW_LINE);
    }	
}

void TBehaviorNetwork::DebugIt2()
{
    int i;
        printf(NEW_LINE);
    for(i = 0; i < iCompIndex; i++) {
	printf("%s\t",pComp[i].GetName());
	for(int j = 0; j < pComp[i].GetEffIndex(); j++)
	    printf("%1.2f ",pComp[i].rEff[j]->GetExp());
        printf(NEW_LINE);
    }
}
