
/*
 * SBC++ (soccer client for Robocup2000)
 * Eslam Nazemi<nazemi@cc.sbu.ac.ir>
 * Amin Abbaspour<a-abbaspour@ce.sbu.ac.ir>
 * Mahmood Rahmani<m-rahmani@ce.sbu.ac.ir>
 * Bahman Radjabalipoor<b-radjabalipoor@ce.sbu.ac.ir>
 * Computer Department
 * Shahid Beheshti University
*/
#include "client.h"
#include "NetFuncs.h"
#include "behave.h"
#include "dribble.h"
#include "pass.h"
#include "NetFuncs.h"
#include "test.h"
#include "types.h"

// minimal dash power (for action parametrization) min 0 max 100
#define MIN_DASH  	   60
#define MAX_SELECTION_TRYS 4
#define DIFF_ANGLE         110
#define MAX_DEFENDER       5 
#define MAX_MIDFIELDER_DEF 2
#define MAX_MIDFIELDER_OFF 2


void DoDeliverMarkData();

/*-----------------------------------------------------------------------------
does nothing
-----------------------------------------------------------------------------*/
void DoNothing()
{ 
}

/*-----------------------------------------------------------------------------
called by the goalie to catch the ball with hands
-----------------------------------------------------------------------------*/
void DoCatchBall()
{
    goalie_catch(Mem->BallAngleFromBody());
}
/*-----------------------------------------------------------------------------
checks, if this agent is in offside
-----------------------------------------------------------------------------*/
void DoCheckOffside()
{
	if(Mem->InOffsidePosition())
		DoRetreat();
	else
		DoRunToHomePosition();
}

/*-----------------------------------------------------------------------------
Covers opponents
-----------------------------------------------------------------------------*/
void DoCover()
{
}

/*-----------------------------------------------------------------------------
Getting open for pass
-----------------------------------------------------------------------------*/
void DoGetOpen()
{ 
 DoMoveToHomePosition()
;
}
/*-----------------------------------------------------------------------------
kicks away the ball. Depending on the position of the agent, the ball is
kicked into goal direction, along the lines or away from the own goal to the side
----------------------------------------------------------------------------*/
void DoKickAwayBall()
{
 smart_kick_hard(Mem->MarkerAngleFromBody(Mem->RM_Their_Goal),KM_Moderate);
}

/*-----------------------------------------------------------------------------
kicks the ball into that half of the goal, where less players are
-----------------------------------------------------------------------------*/
void DoKickGoal()
{
 smart_kick_hard(Mem->MarkerAngleFromBody(Mem->RM_Their_Goal),KM_Moderate);
}

/*-----------------------------------------------------------------------------
turns head into running direction
-----------------------------------------------------------------------------*/
void DoLookStraight()
{
  AngleDeg headAngle =  Mem->MyNeckRelAng();
  if (headAngle != 0)
	turn_neck(-headAngle);
}

/*-----------------------------------------------------------------------------
this could implement a behavior, where the agent has got enough time to
look around every while
-----------------------------------------------------------------------------*/
void DoLookToBall()
{
    // !!! used as long there is no better implementation
    face_only_neck_to_ball();
}

/*-----------------------------------------------------------------------------
Marks an opponent
-----------------------------------------------------------------------------*/
void DoMarkOpponent()
{
    if(Mem->MarkedOpponent > 0 && Mem->MarkedOpponent <= Mem->SP_team_size)
	go_to_point(Mem->OpponentAbsolutePosition(Mem->MarkedOpponent),0.5);
}

/*-----------------------------------------------------------------------------
keeps the eyes pointed to the ball if possible else into rundirection
-----------------------------------------------------------------------------*/
void DoMindBall()
{
    if(!Mem->TurnNeckThisCycle())
        face_only_neck_to_ball();
 // it checks validity
    if(Mem->BallWillBeKickable() && Mem->ViewWidth != VW_Normal) change_view(VW_Narrow);
    else if(!Mem->BallKickable(-0.7) && Mem->ViewWidth != VW_Normal)
 change_view(VW_Normal);
}

/*-----------------------------------------------------------------------------
the goalie can move to its home position after a catch command
-----------------------------------------------------------------------------*/
void DoMoveToHomePosition()
{
	int		ballY = (int) Mem->MyY();
	AngleDeg	dir;
	Unum		teammate;
	Mem->LastCatchTime.t = Mem->CurrentTime.t;
	// move to 5m corner
	if (Mem->MySide == 'r')
	{
		if (ballY > 0)
 {
	     // left top 5m corner
			teammate = 4;
			dir = Mem->MyBodyAng() - 70;
  	        }	
		else
 {
	// left bottom 5m corner
			teammate = 3;
			dir = Mem->MyBodyAng() + 70;
		}
	}
	else
 {
		if (ballY > 0)
 {
 // right top 5m corner
			teammate = 3;
			dir = Mem->MyBodyAng() + 70;
		}
		else
 {
 // right bottom 5m corner
			teammate = 4;
			dir = Mem->MyBodyAng() - 70;
		}
	}
	// then turn into the kickaway direction (remember that the ball is now on the other side)
	turn(dir);
	Mem->GetBall()->set_polar_from_neck(1.0,0.0,Mem->CurrentTime);
	DoPassBall();
}
/*-----------------------------------------------------------------------------
kicks the ball direct to the free teammate
-----------------------------------------------------------------------------*/
void DoPassBall()
{
   smart_kick_hard(Mem->MarkerAngleFromBody(Mem->RM_Their_Goal),KM_Moderate);
}

/*-----------------------------------------------------------------------------
defenders call this to move the offside trap backward
-----------------------------------------------------------------------------*/

void DoPushBackward()
{
 DoMoveToHomePosition()
;
}


/*-----------------------------------------------------------------------------
called to build up an offside trap by defenders
-----------------------------------------------------------------------------*/

void DoPushForward()
{
	int	x  = (int) Mem->MyHomePos().x;
	int	y  = (int) Mem->MyHomePos().y;
	int	x2 = (int) Mem->MyX();
	float     ballX = Mem->BallPositionValid() ? Mem->BallX() : 0;
	int     teammate = Mem->FurthestForwardTeammate();
	float	border;
	int	power;
	float	teammatex;

	if(teammate != Unum_Unknown)
	    teammatex = Mem->TeammateX(teammate);
	else
	    teammatex = 0;
	
	if(Mem->AmIOff()) teammatex = Max(ballX, teammatex);
	else teammatex = Min(ballX, teammatex);

	if (Mem->AmIDef()) 
		border = Min(0, teammatex - 20);
	else
 if (Mem->AmIMid()) 
	    	border = Min(30, teammatex - 10);
	else if (Mem->AmIOff()){
	    teammate = Mem->FurthestBackOpponent();
	    if(teammate != Unum_Unknown && Mem->OpponentPositionValid(teammate)) {
	    	border = Mem->OpponentX(teammate) - 5;
    		if(Mem->MyWing() == PS_Center)  border -= 10;
	    }
	    else {
		border = 10;
    		if(Mem->MyWing() == PS_Center)	border = 0;
	    }
	}			
	    
	if (x2 < border)
		Mem->SetMyHomePos(Vector((x = x2 + 6), y));
	if (Mem->MyStamina() > 2500 )
		power = 60;
	else if (Mem->MyStamina() > 1500)
		power = 40;
	else
		power = 20;
		
	if (go_to_point(Vector(x, y),1, power ,DT_none) == AQ_ActionNotQueued )
	    DoLookToBall();
}

/*-----------------------------------------------------------------------------
does nothing to get more stamina
-----------------------------------------------------------------------------*/
void DoRefreshField()
{
    scan_field_with_body();
}

/*-----------------------------------------------------------------------------
does nothing to get more stamina
-----------------------------------------------------------------------------*/
void DoRelax()
{
    if(Mem->MyNumber == 1)
	DoRunToHomePosition();
    else
	DoLookToBall();
}

/*-----------------------------------------------------------------------------
an offensive player goes backward to leave offside
-----------------------------------------------------------------------------*/
void DoRetreat()
{
	// player is in offside, so go back
	int	x  = (int) Mem->MyHomePos().x;
	int	y  = (int) Mem->MyHomePos().y;
	int	power;

	if (x > -3)
		// agent is on other side
		Mem->SetMyHomePos(Vector((x -= 5), y));

	if ( Mem->MyStamina() > 2500 ) 
		power = 60;
	else
		power = 30;

	// run to (new) home position
	if (go_to_point(Vector(x, y),0, power ,DT_none) == AQ_ActionNotQueued )
		// agent is already there
		DoLookToBall();
}


/*-----------------------------------------------------------------------------
behavior for offenders to keep on the height of the ball
-----------------------------------------------------------------------------*/
void DoRunForwardToBall()
{
	int	x  = (int) Mem->MyHomePos().x;
	int	y  = (int) Mem->MyHomePos().y;
	int	x1 = (int) Mem->MyX();
	int 	x2 = (int) Mem->BallX();
	int	power;

	if ((x2 > x1 + 5) && (x1 < 45))
		// agent is behind ball
		Mem->SetMyHomePos(Vector((x = x1 + 5), y));

	if ( Mem->MyStamina() > 2500 ) 
		power = 60;
	else
		power = 30;

	// run to (new) home position
	test_go_to_point(Vector(x, y), 0, power);
}

/*-----------------------------------------------------------------------------
player runs to the expected position of the ball
-----------------------------------------------------------------------------*/
void DoRunToBall()
{
    test_intercept();
}

/*-----------------------------------------------------------------------------
runs to the current home position of the player. This position may be specified
by other behaviors
-----------------------------------------------------------------------------*/
void DoRunToHomePosition()
{
    if(Mem->CP_goalie && Mem->BallPositionValid()){
	Line l = LineFromTwoPoints(Mem->MarkerPosition(Mem->RM_My_Goal),Mem->BallAbsolutePosition());
	Vector intersection= AdjustPtToRectOnLine(Mem->BallAbsolutePosition(),Mem->OwnGoalieArea,l);
	Mem->LogAction2(40, "Test_Goalie: moving around Goalie area");
	go_to_point(Vector(Max(intersection.x, -50), intersection.y) ,0.2);
	return;
    }
    else if (go_to_point(Mem->MyPosGoalKick(),1, 100 ,DT_none) == AQ_ActionNotQueued)
	// agent is already there
	face_neck_to_ball();    
}

/*-----------------------------------------------------------------------------
searches the ball by looking around and changing view angle to 180
-----------------------------------------------------------------------------*/
void DoSearchBall()
{
    scan_field_with_body();
}


/*-----------------------------------------------------------------------------
the player does a dribbling
-----------------------------------------------------------------------------*/
void DoRunWithBall()
{
    smart_kick_hard(Mem->MarkerAngleFromBody(Mem->RM_Their_Goal),KM_Moderate);
}


/*-----------------------------------------------------------------------------
say the own position and that of the valid ball and all valid players
-----------------------------------------------------------------------------*/
void DoSayPosition()
{
    say_position();
}


/*-----------------------------------------------------------------------------
the player runs behind the ball if playmode is KICK_IN
-----------------------------------------------------------------------------*/
void DoTurnAroundBallForKickIn()
{
    test_go_to_static_ball();
}

void DoDeliverMarkData()
{
    Unum *Players;
    Players = new Unum [Mem->SP_team_size];
    int result = Mem->SortPlayersByXToPoint('t'/*Mem->TheirSide*/,
					Mem->MyPos(),Players);
    char s[30],s2[3];
    int  mn = 0;

    if(result > 0) {
	for(int i = 0; i < Mem->SP_mark_size; i++) 
	    if( (Players[i] > 0) && (Players[i] <= Mem->SP_team_size)) mn++;

	sprintf(s,"(m %d", mn);

	for(int i = 0; i < Mem->SP_mark_size; i++) 
	    if( (Players[i] > 0) && (Players[i] <= Mem->SP_team_size)) {
		sprintf(s2," %d ",Players[i]);
		strcat(s,s2);
	    }
	strcat(s," )");
	say(s);
    }
    delete [] Players;
}

Unum GetOpponentInWay(float dir)
{
    float minDir = 1000.0;
    float minDist ;
    float dist ;
    Unum  indexMinDir = -1;
    Unum  opp;
    float deltaDir;

    for ( opp = 1; opp <= Mem->SP_team_size ; opp++) {
	if(Mem->OpponentPositionValid(opp)) {
    	    dist = Mem->OpponentDistance(opp);
	    if(dist < 5.0) {
		deltaDir = dir - Mem->OpponentAngleFromBody(opp);
		if( deltaDir < 0)
		    deltaDir = -deltaDir;
		if( deltaDir < minDir ) {
		    minDir = deltaDir;
		    minDist = dist;
	    	    indexMinDir = opp;
		}
	    }
	}
   }

   if ( minDir < 90 && (minDist * sin (Deg2Rad(minDir))) < 3.0 )
      return opp;
   return Unum_Unknown;
}

