
/* NetFuncs.C
 * SBC++ (soccer client for Robocup2000)
 * Eslam Nazemi<nazemi@cc.sbu.ac.ir>
 * Amin Abbaspour<a-abbaspour@ce.sbu.ac.ir>
 * Mahmood Rahmani<m-rahmani@ce.sbu.ac.ir>
 * Bahman Radjabalipoor<b-radjabalipoor@ce.sbu.ac.ir>
 * Computer Department
 * Shahid Beheshti University
*/

#include "NetFuncs.h"
#include "utils.h"
#include "client.h"
#include "MemPlayer.h"

#include <stdio.h>
#include <ncurses.h>

TNetCommand::TNetCommand()
{
    char    str[STRLEN];
    FILE    *in = fopen(COMMANDS_LIST_FILE, "r");
    if(!in) {
	my_error("Unable to open commands list file.\n");
	_exit(0);
    }

    for(int i = 0; i< MaxCommands; i++) {
	Command[i] = new TCommandsList;
	if(!Command[i]) {
	    my_error("TNetCommand : Cann't allocate mem 4 commnad.\n");
	    _exit(0);
	}
	fscanf(in, "%s" ,str);
	down(str);
	Command[i]->SetName(str);
    }
    fclose(in);
    InitFastFuncs();
}

int TNetCommand::IsValid(const char *s)
{
    char *s2 = strdup(s);
    down(s2);
    for(int i = 0; i < MaxCommands; i++)
        if( !strcmp(s2, Command[i]->GetName()) ) {
	    Command[i]->SetUsed(TRUE);
	    return i;
	}
    my_error("Unkown Command : %s",s2);
    _exit(0);
    return 0;
}

TNetBehave::TNetBehave()
{
    char  str[STRLEN];
    FILE *in = fopen(BEHAVES_LIST_FILE, "r");
    if(!in) {
	my_error("Unable to open behaves list file\n");
	_exit(0);
    }

    for(int i = 0; i < MaxBehaves; i++) {
	fscanf(in, "%s", str);
	down(str);
	strcpy(sBehaveList[i], str);
    }
    fclose(in);
    InitFastBehaves();
}

int TNetBehave::IsValid(const char *s)
{
    char *s2 = strdup(s);
    down(s2);
    for(int i = 0; i < MaxBehaves; i++)
	if( !strcmp(s2, sBehaveList[i] ) ) return i;
    my_error("Unkown Behave : ",s2);
    _exit(0);
    return 0;
}

/**************************** F O R   D E B U G ******************************/
/**************************** F O R   D E B U G ******************************/
/**************************** F O R   D E B U G ******************************/

void _DoNothing()			
{
    Mem->LogAction2(5, "Nothing");
    DoNothing();
}

void _DoCatchBall()
{
    Mem->LogAction2(5, "Catch Ball");	
    DoCatchBall();
}

void _DoCover()
{
    Mem->LogAction2(5, "LEG -> Cover");
    DoCover();
}

void _DoGetOpen()
{
    Mem->LogAction2(5, "LEG -> Getting open for pass");
    DoGetOpen();
}

void _DoCheckOffside()		
{
    Mem->LogAction2(5, "LEG -> Check Offside");	
    DoCheckOffside();		
}

void _DoKickAwayBall()
{
    Mem->LogAction2(5, "LEG -> Kick Away Ball");	
    DoKickAwayBall();
}

void _DoKickGoal()
{
    Mem->LogAction2(5, "LEG -> Kick Goal");	
    DoKickGoal();
}

void _DoMarkOpponent()
{
    Mem->LogAction2(5, "LEG -> Mark");
    DoMarkOpponent();
}

void _DoMindBall()
{
    Mem->LogAction2(5, "NECK -> Mind Ball");	
    DoMindBall();
}

void _DoMoveToHomePosition()
{
    Mem->LogAction2(5, "Move To Home Position");
    DoMoveToHomePosition();
}	

void _DoLookStraight()
{
    Mem->LogAction2(5, "NECK -> Look Stright");	
    DoLookStraight();
}

void _DoPassBall()
{
    Mem->LogAction2(5, "LEG -> Pass Ball");	
    DoPassBall();
}

void _DoPushBackward()
{
    Mem->LogAction2(5, "LEG -> Push Backward");
    DoPushBackward();
}

void _DoPushForward()
{
    Mem->LogAction2(5, "LEG -> Push Forward");	
    DoPushForward();
}

void _DoRefreshField()
{
    Mem->LogAction2(5, "NECK -> Refresh");	
    DoRefreshField();
}

void _DoRelax()
{
    Mem->LogAction2(5, "LEG -> Relax");	
    DoRelax();
}

void _DoRetreat()
{
    Mem->LogAction2(5, "LEG -> Retreat");	
    DoRetreat();
}

void _DoRunForwardToBall()
{
    Mem->LogAction2(5, "LEG -> Run Forward To Ball");
    DoRunForwardToBall();
}

void _DoRunToBall()	
{
    Mem->LogAction2(5, "LEG -> Run To Ball");
    DoRunToBall();
}

void _DoRunToHomePosition()
{
    Mem->LogAction2(5, "LEG -> Run To Home Position");  	
    DoRunToHomePosition();
}

void _DoRunWithBall()
{
    Mem->LogAction2(5, "LEG -> Run With Ball");
    DoRunWithBall();
}

void _DoSayPosition()
{
    Mem->LogAction2(5, "MOUTH -> Say Position");
    DoSayPosition();
}

void _DoSearchBall()
{
    Mem->LogAction2(5, "LEG -> Search Ball");
    DoSearchBall();
}

void _DoTurnAroundBallForKickIn() 	
{
    Mem->LogAction2(5, "LEG -> Turn Around Ball");	
    DoTurnAroundBallForKickIn();
}

/**************************** F O R   D E B U G ******************************/
/**************************** F O R   D E B U G ******************************/
/**************************** F O R   D E B U G ******************************/

void TNetCommand::InitFastFuncs() 
{
	FastFunc[ nothing 		] =  GetNothing;
	FastFunc[ ballahead 		] =  GetBallAhead;
	FastFunc[ ballbehind 		] =  GetBallBehind;
	FastFunc[ balliscatchable 	] =  GetBallIsCatchable;
	FastFunc[ ballisfar 		] =  GetBallIsFar;
	FastFunc[ ballisnearothergoal 	] =  GetBallIsNearOtherGoal;
	FastFunc[ ballisnearowngoal 	] =  GetBallIsNearOwnGoal;
	FastFunc[ ballisverynearowngoal	] =  GetBallIsVeryNearOwnGoal;
	FastFunc[ ballkickable		] =  GetBallKickable;
	FastFunc[ balloutside 		] =  GetBallOutside;
	FastFunc[ catchsuccess		] =  GetCatchSuccess;
	FastFunc[ freeteammateisoffside	] =  GetFreeTeammateIsOffside;
	FastFunc[ haveenoughstamina 	] =  GetHaveEnoughStamina;
	FastFunc[ highstamina 		] =  GetHighStamina;
	FastFunc[ inoffside 		] =  GetInOffside;
	FastFunc[ inotherhalf 		] =  GetInOtherHalf;
	FastFunc[ justcaughtball 	] =  GetJustCaughtBall;
	FastFunc[ knowmarkedopponent 	] =  GetKnowMarkedOpponent;
	FastFunc[ knownotinoffside 	] =  GetKnowNotInOffside;
	FastFunc[ lowstamina 		] =  GetLowStamina;
	FastFunc[ maysaysomething 	] =  GetMaySaySomething;
	FastFunc[ mygoalkick	 	] =  GetMyGoalKick;
	FastFunc[ nearball 		] =  GetNearBall;
	FastFunc[ neargoal 		] =  GetNearGoal;
	FastFunc[ nearopponent 		] =  GetNearOpponent;
	FastFunc[ nearowngoal 		] =  GetNearOwnGoal;
	FastFunc[ offsideposition	] =  GetOffsidePosition;
	FastFunc[ opponentisfree  	] =  GetOpponentIsFree;
	FastFunc[ opponentinfront 	] =  GetOpponentInFront;
	FastFunc[ playmodeon 		] =  GetPlayModeOn;
	FastFunc[ standfree 		] =  GetStandFree;
	FastFunc[ teammatebehind 	] =  GetTeammateBehind;
	FastFunc[ teammateinfront 	] =  GetTeammateInFront;
	FastFunc[ teammateisbetweenball	] =  GetTeammateIsBetweenBall;
	FastFunc[ teammateisfree 	] =  GetTeammateIsFree;
	FastFunc[ teammateisnearerball 	] =  GetTeammateIsNearerBall;
	FastFunc[ teammatesinplaces 	] =  GetTeammatesInPlaces;
	FastFunc[ theirgoalkick 	] =  GetTheirGoalKick;

	FastFunc[ haveball	     	] =  GetHaveBall;
	FastFunc[ havemarked	     	] =  GetHaveMarked;
	FastFunc[ haveseenball 		] =  GetHaveSeenBall;
	FastFunc[ losestamina  		] =  GetLoseStamina;
	FastFunc[ noothergoal  		] =  GetNoOtherGoal;
	FastFunc[ offsidetrap  		] =  GetOffsideTrap;
	FastFunc[ soccergoal   		] =  GetSoccerGoal;
	FastFunc[ teammatehasball	] =  GetTeammateHasBall;
}

void TNetBehave::InitFastBehaves()
{

	FastBehave [ nothing			] = _DoNothing;
	FastBehave [ catchball			] = _DoCatchBall;
	FastBehave [ checkoffside		] = _DoCheckOffside;
	FastBehave [ cover        		] = _DoCover;
	FastBehave [ getopen        		] = _DoGetOpen;
	FastBehave [ kickawayball		] = _DoKickAwayBall;
	FastBehave [ kickgoal			] = _DoKickGoal;
	FastBehave [ lookstraight		] = _DoLookStraight;
	FastBehave [ markopponent		] = _DoMarkOpponent;
	FastBehave [ mindball			] = _DoMindBall;
	FastBehave [ movetohomeposition		] = _DoMoveToHomePosition;
	FastBehave [ passball			] = _DoPassBall;
	FastBehave [ pushbackward		] = _DoPushBackward;
	FastBehave [ pushforward		] = _DoPushForward;
	FastBehave [ refreshfield		] = _DoRefreshField;
	FastBehave [ relax			] = _DoRelax;
	FastBehave [ retreat			] = _DoRetreat;
	FastBehave [ runforwardtoball		] = _DoRunForwardToBall;
	FastBehave [ runtoball			] = _DoRunToBall;
	FastBehave [ runtohomeposition		] = _DoRunToHomePosition;
	FastBehave [ runwithball		] = _DoRunWithBall;
	FastBehave [ sayposition		] = _DoSayPosition;
	FastBehave [ searchball			] = _DoSearchBall;
	FastBehave [ turnaroundballforkickin	] = _DoTurnAroundBallForKickIn;
/*
	FastBehave [ nothing			] = DoNothing;
	FastBehave [ catchball			] = DoCatchBall;
	FastBehave [ checkoffside		] = DoCheckOffside;
	FastBehave [ cover        		] = DoCover;
	FastBehave [ getopen        		] = DoGetOpen;
	FastBehave [ kickawayball		] = DoKickAwayBall;
	FastBehave [ kickgoal			] = DoKickGoal;
	FastBehave [ lookstraight		] = DoLookStraight;
	FastBehave [ markopponent		] = DoMarkOpponent;
	FastBehave [ mindball			] = DoMindBall;
	FastBehave [ movetohomeposition		] = DoMoveToHomePosition;
	FastBehave [ passball			] = DoPassBall;
	FastBehave [ pushbackward		] = DoPushBackward;
	FastBehave [ pushforward		] = DoPushForward;
	FastBehave [ refreshfield		] = DoRefreshField;
	FastBehave [ relax			] = DoRelax;
	FastBehave [ retreat			] = DoRetreat;
	FastBehave [ runforwardtoball		] = DoRunForwardToBall;
	FastBehave [ runtoball			] = DoRunToBall;
	FastBehave [ runtohomeposition		] = DoRunToHomePosition;
	FastBehave [ runwithball		] = DoRunWithBall;
	FastBehave [ sayposition		] = DoSayPosition;
	FastBehave [ searchball			] = DoSearchBall;
	FastBehave [ turnaroundballforkickin	] = DoTurnAroundBallForKickIn;
*/
}