
/* NetNodes.C
 * SBC++ (soccer client for Robocup2000)
 * Eslam Nazemi<nazemi@cc.sbu.ac.ir>
 * Amin Abbaspour<a-abbaspour@ce.sbu.ac.ir>
 * Mahmood Rahmani<m-rahmani@ce.sbu.ac.ir>
 * Bahman Radjabalipoor<b-radjabalipoor@ce.sbu.ac.ir>
 * Computer Department
 * Shahid Beheshti University
*/
 
#include "NetNodes.h"
#include "NetHeader.h"
#include "NetParams.h"
#include "client.h"

#include <iostream.h>

TCompetence::TCompetence() : TNetNode()
{
    SetExecuted(FALSE);
    SetGama(DEFAULT_GAMA);
    SetPrevAct(0.0);
    SetAct(0.0);

    int i;
    for(i = 0; i < MaxPreConditions; i++) rPre[i] = new TFunction;
    for(i = 0; i < MaxEffects; i++) rEff[i] = new TFunction;
    for(i = 0; i < MaxExpantions; i++) 	rExp[i] = new TFunction;
    for(i = 0; i < MaxGoals; i++) fActOfGoal[i] = fPrevActOfGoal[i] = -1000.0;
	
    iResource = RESOURCE_LEG;
    iPreIndex = 0;
    iEffIndex = 0;
    iExpIndex = 0;
    iGoalIndex = 0;
}

TCompetence::~TCompetence()
 
{
    int i;
    for(i = 0; i < MaxPreConditions; i++) delete rPre[i];
    for(i = 0; i < MaxEffects; i++) delete rEff[i];
    for(i = 0; i < MaxExpantions; i++) delete rExp[i];
 

}

void TCompetence::AddCondition(const int& i, const float& val)
{
    if(iPreIndex >= MaxPreConditions) {
	my_error("AddCondition : Cann't add more PreConditions.\n");
	_exit(0);
    }
    rPre[iPreIndex]->SetFunc(i);
    rPre[iPreIndex]->SetExp(val);
    iPreIndex++;
}

void TCompetence::AddEffect(const int& i, const float& val)
{
    if(iEffIndex >= MaxEffects) {
	my_error("AddEffect : Cann't add more Effects.\n");
	_exit(0);
    }
    rEff[iEffIndex]->SetFunc(i);
    rEff[iEffIndex]->SetExp(val);
    iEffIndex++;
    if( i > GoalCommandStart )
	iGoalIndex++;
}

void TCompetence::AddExpantion(const int& i, const int& j, const float& val)
{
    if(iExpIndex >= MaxExpantions) {
	my_error("AddExpantion : Cann't add more Expantions.\n");
	_exit(0);
    }
    rExp[iExpIndex]->iComp = i;
    rExp[iExpIndex]->SetFunc(j);
    rExp[iExpIndex]->SetExp(val);
    iExpIndex++;
}

void TCompetence::SetResource(const char *res)
{
    char *s = strdup(res);
    down(s);
    if( !strcmp(s, STRING_LEG) )
 iResource = RESOURCE_LEG;
    else if( !strcmp(s, STRING_MOUTH))
 iResource = RESOURCE_MOUTH;
    else if( !strcmp(s, STRING_NECK)) iResource = RESOURCE_NECK;
    delete s;
}


