
/* pass.C
 * SBC++ (soccer client for Robocup2000)
 * Eslam Nazemi<nazemi@cc.sbu.ac.ir>
 * Amin Abbaspour<a-abbaspour@ce.sbu.ac.ir>
 * Mahmood Rahmani<m-rahmani@ce.sbu.ac.ir>
 * Bahman Radjabalipoor<b-radjabalipoor@ce.sbu.ac.ir>
 * Computer Department
 * Shahid Beheshti University
*/

#include "client.h"
#include "pass.h"
#include <math.h>
#include "types.h"
#include "utils.h"

#define  EPSILON 0.001
#define  PassTh	 0.1

/*-----------------------------------------------------------------------------
Function: Unum CalculateFreestTeammate()
-----------------------------------------------------------------------------*/
Unum  CalculateFreestTeammate()
{
	Unum	i;
	float	value;
	float	fHighest = PassTh;
	int	iHighest = -1;

	for (i = 2; i <= Mem->SP_team_size; i++)
 /* not goalie */
	{
	    if ( Mem->TeammatePositionValid(i) && (i != Mem->MyNumber) )
	    {	

		value = CalculateFreePlayer(i);
		if (value > fHighest)
		{
		    fHighest = value;
		    iHighest = i;
		}
	    }
	}

	if (iHighest < 0)
		return Unum_Unknown;
	else 
		Mem->PassDegree = fHighest;
		return iHighest;
}

/*-----------------------------------------------------------------------------
Function: float	CalculateFreePlayer (Unum pTeammate)
-----------------------------------------------------------------------------*/
float	CalculateFreePlayer (Unum pTeammate)
{
    int		i;
    AngleDeg	fSecureBoundary = 20;
    float	fQuality;
    float	fQualityOfPossibility;		// 0.5
    float	fQualityOfPassDir;	// 0.2
    float	fQualityOfMateDistance;	// 0.1
    float	fQualityOfGoalDistance;
    float	fDistanceToOwnGoal;
    AngleDeg	fFreeAreaBordersLeft ;
    AngleDeg	fFreeAreaBordersRight;
    float	fDist;
    AngleDeg	fWidth;
    Vector	tOptimalPassPoint;
    int		iDeltaStart;
    int		iDeltaStop;

    float	fTeammateDistance      = Mem->TeammateDistance(pTeammate);
    AngleDeg	fTeammateAngleFromBody = Mem->TeammateAngleFromBody(pTeammate);

    if (fDist < TMATE_MAX_DIST)
    	fQualityOfMateDistance = GetLinearFuzzyValue(fTeammateDistance, TMATE_MIN_DIST, TMATE_MID_DIST);
    else
	fQualityOfMateDistance = GetLinearFuzzyValue(fTeammateDistance, TMATE_MAX_DIST, MAX_SHOT_RANGE, FALSE);
    
    if 
(fQualityOfMateDistance < EPSILON)
        return 0.0;

    if(Mem->CP_goalie)
	fQualityOfMateDistance = 1; 	/* for goal start */
	
    //set quality of distance to own goal
    fDistanceToOwnGoal = Mem->TeammateDistanceTo(pTeammate, Mem->MarkerPosition(Mem->RM_My_Goal));
    fQualityOfGoalDistance = GetLinearFuzzyValue(fDistanceToOwnGoal, 15.0, 35.0);
    if (fQualityOfGoalDistance < EPSILON)
        return 0.0;
    
    //calculate quality of direction to the other goal
    iDeltaStart = 50;
    iDeltaStop = 80;
    GetOptimalPassPoint(tOptimalPassPoint, iDeltaStart, iDeltaStop);
    fWidth = fabs(fTeammateAngleFromBody - Mem->AngleToFromBody(tOptimalPassPoint));
    fQualityOfPassDir = GetLinearFuzzyValue(fWidth, iDeltaStart, iDeltaStop, FALSE);
    if (fQualityOfPassDir < EPSILON )
        return 0.0;
	    
    //calculate quality of possibility

    if (fTeammateDistance < MAX_SHOT_RANGE  )
 { 

	fFreeAreaBordersLeft = fTeammateAngleFromBody - fSecureBoundary;
	fFreeAreaBordersRight= fTeammateAngleFromBody + fSecureBoundary; 

	for (i = 1; i <= Mem->SP_team_size; i++)
 {

	    if (Mem->OpponentPositionValid(i) && 
		    (Mem->OpponentDistance(i) - 3 < fTeammateDistance)){
    		
		AngleDeg fOpponentAngleFromBody = Mem->OpponentAngleFromBody(i);
		
		if (fOpponentAngleFromBody < fTeammateAngleFromBody &&
 
		     fOpponentAngleFromBody > fFreeAreaBordersLeft )
			fFreeAreaBordersLeft  = fOpponentAngleFromBody ;
    		else 
		    if (fOpponentAngleFromBody < fFreeAreaBordersRight)
			fFreeAreaBordersRight = fOpponentAngleFromBody;
	    }
	}

	float fFreePassArea = fFreeAreaBordersRight - fFreeAreaBordersLeft;
	fQualityOfPossibility = fFreePassArea / (fSecureBoundary * 2.0);

    }
    else
        return 0.0; 
	
	fQuality = fQualityOfPossibility * fQualityOfPassDir * 
		    fQualityOfMateDistance
 * fQualityOfGoalDistance;

	fQuality = sqrt(sqrt(fQuality));
	return fQuality;
}

/*-----------------------------------------------------------------------------
Function: void GetOptimalPassPoint(Vector point)
-----------------------------------------------------------------------------*/

void GetOptimalPassPoint(Vector& point, int& iDeltaStart, int& iDeltaStop)
{
	int	x = (int) Mem->MyX();
	int	y = (int) Mem->MyY();

	if (x < -30)
	{
		if (y > 0)
			point = Vector(-20, 20);
		else 
			point = Vector(-20, -20);

		iDeltaStart = 30;
		iDeltaStop = 70;
	}


	else if (x < -15)
	{
		// near the own goal, the player should pass to one of the midline posts
		if (y > 5)
			point = Vector(0,30);
		else 
if (y < -5)
			point = Vector(0,-30);
		else 
			point = Vector(0,0);

		iDeltaStart = 50;
		iDeltaStop = 80;
	}
	else if (x < 15)
	{
		// near the own goal, the player should pass to one of the midline posts
		if (y > 0)
			point = Vector(25,30);
		else 
			point = Vector(25,-30);

		iDeltaStart = 50;
		iDeltaStop = 80;
	}
	else if (x < 30) 
	{
	    if (y > 5)
		point = Vector(40,30);
	    else 
if (y < -5)
		point = Vector(40,-30);
	    else 
		point = Vector(40,0);
	    // we are close to the goal, pass to 11m point
	    iDeltaStart = 50;
	    iDeltaStop = 80;
	}
	else if (x < 40) 
	{
	    if (y > 15)
		point = Vector(50,20);
	    else 
if (y < -15)
		point = Vector(50,-20);
	    else 
		point = Mem->MarkerPosition(Mem->RM_Their_PC_Flag);
;

	    iDeltaStart = 50;
	    iDeltaStop = 80;
	}
	else
	{
	    if (y > 10 || y < -10)
		point = Mem->MarkerPosition(Mem->RM_Their_PC_Flag);
	    else
		point = Mem->MarkerPosition(Mem->RM_Their_Goal);

	    iDeltaStart = 50;
	    iDeltaStop = 80;
	}
}

